@@ -1,5 +1,5 @@
 package Geo::Address::Formatter;
-$Geo::Address::Formatter::VERSION = '1.994';
+$Geo::Address::Formatter::VERSION = '1.995';
 # ABSTRACT: take structured address data and format it according to the various global/country rules
 
 use strict;
@@ -39,6 +39,8 @@ sub new {
     $debug         = (defined($params{debug})        && $params{debug})        // 0;
 
     $self->{final_components} = undef;
+    $self->{set_district_alias} = {};
+
     bless($self, $class);
 
     say STDERR "************* in Geo::Address::Formatter::new ***" if ($debug);
@@ -882,6 +884,14 @@ sub _set_district_alias {
     my $self = shift;
     my $cc = shift;
 
+    # this may get called repeatedly
+    # no need to do the work again
+    if (defined($cc)){
+        my $ucc = uc($cc);
+        return if (defined($self->{set_district_alias}{$ucc}));
+        $self->{set_district_alias}{$ucc} = 1;
+    }
+
     my $oldalias;
     if (defined($small_district{$cc})){
         $self->{component2type}{district} = 'neighbourhood';
@@ -903,7 +913,6 @@ sub _set_district_alias {
     # remove from the old alias list
     my @temp = grep { $_ ne 'district' } @{$self->{component_aliases}{$oldalias}};
     $self->{component_aliases}{$oldalias} = \@temp;
-
     return;
 }  
 
@@ -914,8 +923,7 @@ sub _find_unknown_components {
     my $rh_components = shift;
 
     my %h_known   = map  { $_ => 1 } @{$self->{ordered_components}};
-    my @a_unknown = grep { !exists($h_known{$_}) } sort keys %$rh_components;
-
+    my @a_unknown = grep { !exists($h_known{$_}) } keys %$rh_components;
     return \@a_unknown;
 }
 
@@ -933,7 +941,7 @@ Geo::Address::Formatter - take structured address data and format it according t
 
 =head1 VERSION
 
-version 1.994
+version 1.995
 
 =head1 SYNOPSIS
 
